#include <msgServer.hh>

using namespace std;

msgServer::msgServer(const std::string& address, const std::string& port, bool toggleTime, unsigned long int inact, size_t maxConnections) {
	this->inactivityTime = inact;
	this->isRunning = iserv.createServer(address, port, maxConnections);
	isMessagesWaiting = false;
	if(this->isRunning) {
		if(toggleTime) iserv.setInactivityTime(this->inactivityTime);
		cout << "----------------------" << endl;
		cout << "msgconnector Server in now running on port " << address << ":" << port << endl;
		cout << " - max authorized connections: " << maxConnections << endl;
		if(toggleTime)
			cout << " - max inactivity time: " << this->inactivityTime << endl;
		else
			cout << " - max inactivity time: unlimited" << endl;
		cout << "----------------------" << endl;		
	}else{
		cout << "MsgServer could not start." << endl;
	}
}

//void msgServer::setInactivityTime(unsigned int inact) {
//	if(this->isRunning) {
//		iserv.setInactivityTime(this->inactivityTime);
//	}
//}

void msgServer::addConnection(int clientId, string clientName) {
	clients.insert(std::make_pair(clientId, clientName));
	cout << clientName << " is now connected (id=" << clientId << ")" << endl;
}

void msgServer::removeConnection(int c) {
	if(clients.find(c)!=clients.end()){
		string n = clients[c];
		clients.erase(clients.find(c));
		cout << n << " is disconnected (id=" << c << ")" << endl;
	}
}

void msgServer::manageIncomingMessage(int c, message m) {
	string dest = m.getStringHead();
	string content = m.getStringBody();
	if(dest == DEFAULT_SERVER_NAME) { // command
		this->addConnection(c, content);
	} else if(dest == DEFAULT_BROADCAST_NAME) { //message to broadcast
		string from = clients[c];
		message temp;
		
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			if(it->first!=c){
				int client = it->first;
				temp.encode(from.c_str(), content.c_str());
				if(!iserv.send(client, temp)) std::cerr<<"ERROR: Could not send"<<endl;
			}
		}
	} else { // transfer
		string from = clients[c];
		message temp;
		temp.encode(from.c_str(), content.c_str());
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			if(it->second==dest){
				int client = it->first;
				if(!iserv.send(client, temp)) std::cerr<<"ERROR: Could not send"<<endl;
			}
		}
	}
}

void msgServer::run() {
	while(this->isRunning) {
		vector< pair<iserver::status, int> > events = iserv.waitReadEvents();
		std::vector< pair<iserver::status, int> >::const_iterator it;
		for(it=events.begin(); it!=events.end(); ++it){
			const pair<iserver::status, int>& event = *it;
			if(event.first == iserv.READ_EVENT && event.second >= -1) {
				pair<iserver::received, message> recpt = iserv.receive(event.second);
				if(recpt.first == iserv.STAT_ERR) this->removeConnection(event.second);
				else this->manageIncomingMessage(event.second, recpt.second);
			}
			if(event.first==iserver::ERROR_EVENT && event.second==0){
				std::cerr<<"Shut down the server due to inactivity"<<std::endl;
				this->shutdown();
			}
		}
	}
}

void msgServer::shutdown() {
	if(this->isRunning) {
		//Broadcast a message telling the server is quitting
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			int client = it->first;
			message temp;
			temp.encode(DEFAULT_SERVER_NAME, SERVER_EXIT);
			if(!iserv.send(client, temp)) std::cerr<<"ERROR: Could not send"<<endl;
		}
		this->isRunning = false;
		iserv.stopServer();
	}
}

msgServer::~msgServer() {
	if(this->isRunning){
		cout << "shuting down server... " << endl;
	}
	this->shutdown();
}

